#!/bin/sh
#################################################################################################
# Slackware update script									#
#												#
# Description:  This script was written to keep a slackware system up to date with the patches	#
#		directory of a given release.  This script should probably not be used on a	#
#		slackware-current system unless you really know what you are doing.  There are 	#
#		3 runlevels available. 0 and 1 are both safe and will not do anything to your	#
#		system.  2 can be dangerous depending on what you are updating.  		#
#												#
# Supports:	Slackware 8.1 - 9.1 and slackware-current					#
#												#
# Requirements: cat cp cut echo grep ls md5sum mkdir pwd rm rmdir sed tr wc wget which          #
#		(all of these should be on a slackware system by default)			#
#												#
# Contact Info:	Rob Hunter <slackware[AT]vt[D0T]edu>						#
#################################################################################################

#################
# Configuration #
#################
Server1="ftp://carroll.cac.psu.edu/pub/linux/distributions/slackware"
Server2="ftp://distro.ibiblio.org/pub/Linux/distributions/slackware"
Server3="ftp://ftp.slackware.com/pub/slackware"

 		# FireWall / Proxy Configuration (Not Enabled by Default)
#http_proxy=""	# e.g. http_proxy="http://proxy:3128/"
#ftp_proxy=""	# e.g. ftp_proxy="http://proxy/proxy.pac"

Passiv=1	# Passive:	0 - Disable Passive transfers
		#		1 - Enable Passive transfers

RunLevel=1	# RunLevels:	0 - Only Check for Updates
		# 		1 - Check for and Download Updates [Recomended]
		#		2 - Automaticly Install Updates (Be carefull with this one)
		
Verbose=0	# verbosity:	0 - Only show updated packages
		#		1 - Show status of all patches found on server

VersionCheck=1	# VersionCheck: 0 - Do not check for a newer version of slackupdate
		#		1 - Check for a new slackupdate [Default]

TEMPDIR="/tmp/slackupdate"	# Temporary Directory
				# (WARNING ALL CONTENT IS ERASED!)
##################-[You do not need to configure anything below this line]-##################

###########
# Globals #
###########
VERSION="Slackware Update Version [0.6.2]"
HELP="
Usage: slackupdate [-l (0|1|2)] [-o /directory] [-r] [-s server] [-t /directory] [-u version] [-h] [-v]

  -l (0|1|2)	Runlevel, Overrides the default 
		(0 - check | 1 - Download | 2 - Automatic)
  -o (/dir)	Output Directory, Place to put downloaded packages 
  		e.g. [-o ~/slackupdate]
  -r		Retry, Will not re-downloading updates you already have.
  -t (/dir)	Temporary Directory [-t /tmp/slackupdate]
  -s (server)	Preferred server e.g. [-s ftp://ftp.slackware.com/pub/slackware]
  -u (version)	Upgrade to the specified version of slackware !!!EXPERIMENTAL!!!
  -v		More verbosity
  -h		This help dialog.  May also use (-help|--help|-?)
  
[Runtime examples]
slackupdate.sh				# Download updates for your system
slackupdate.sh				# Download updates for slackware-current
slackupdate.sh -r -u slackware-9.1	# Download upgrade to slackware-9.1
slackupdate.sh -r -l 2 -u slackware-9.1	# Upgrade to slackware-9.1
"
COMMANDS="cat cp cut echo grep ls md5sum mkdir pwd rm rmdir sed tr wc wget which /sbin/upgradepkg"
NoUpdates=1
Patches="patches"
Retry=0

if [ $Passiv == 1 ]; then
  wgetOptions="-q -t 5 --passive-ftp"
else
  wgetOptions="-q -t 5"
fi

#############
# Functions #
#############

# Define exit/failure function.
exit_failure() {
  if [ ! -z "$FAILURE" ]; then
    echo -e "\\033[1;31m [ FAILED ]"
    echo "-> FATAL: $FAILURE"
    echo -e "-> ** ABORTED **.\\033[0;00m"
    exit 1
  else
    exit 0
  fi
}

# Checks for a newer version of this script
check_for_new_version() {
  if [ "$VersionCheck" == 1 ]; then
    echo -n "Checking for a newer version of slackupdate.sh "
    wget $wgetOptions http://128.173.184.249/slackupdate/files/version 1> /dev/null 2> /dev/null
    if [ -r version ]; then
      NVERSION=`cat version`
      if [ "$NVERSION" == "$VERSION" ]; then
        echo -e "[\\033[36;40mUp2Date\\033[0;00m]"
      else
        echo
        echo -e "\\033[36;40m$NVERSION\\033[0;00m is available"
        echo "Download from http://128.173.184.249/slackupdate/files/slackupdate.sh"
      fi
    else
      echo -e "[\\033[1;31mserver down\\033[0;00m]"
    fi
  fi
}

# check to make sure all the commands are there
check_for_commands() {
  for COMMAND in $COMMANDS; do
    which $COMMAND 1> /dev/null 2> /dev/null
    if [ $? != 0 ]; then
      FAILURE="$COMMAND not found.  please install $COMMAND first"
      exit_failure
    fi
  done
}

# Remove the temp directory
clean_tempdir() {
  if [ -d $TEMPDIR ]; then
    cd "$TEMPDIR"
    TEMPDIR=`pwd`
    if [ -w $TEMPDIR ]; then
      rm *.md5 version* CHECKSUMS* 1> /dev/null 2> /dev/null
      if [ $Retry == 0 ]; then
        rm *.tgz 1> /dev/null 2> /dev/null
      fi
      rmdir $TEMPDIR 1> /dev/null 2> /dev/null      
    else
      FAILURE="You do not have write permissions for $TEMPDIR"
      exit_failure
    fi
  elif [ -r "$TEMPDIR" ];then
    FAILURE="Invalid Temp Directory: $TEMPDIR "
    exit_failure
  fi
}

# Copy packages to output directory
copy_output() {
  if [ ! -z "$OUTPUTDIR" ]; then
    cd "$ORGDIR"
    if [ ! -d "$OUTPUTDIR" ]; then
      mkdir "$OUTPUTDIR" 1> /dev/null 2> /dev/null
      if [ $? != 0 ]; then
        FAILURE="Could not create directory $OUTPUTDIR"
        exit_failure
      fi  
    fi
    cd "$OUTPUTDIR"
    OUTPUTDIR=`pwd`
    cp $TEMPDIR/*.tgz $OUTPUTDIR
  fi
  clean_tempdir
}

# Download new package
download_package() {
  echo -n "$PackageName => $TrueUpdateName"
  if [ "$RunLevel" -ge  1 ]; then
    if [ ! -r $TEMPDIR/$TrueUpdateName.tgz ]; then
      echo -e -n " [\\033[1;32mDownloading\\033[0;00m]"
      wget $wgetOptions $Server/$Patches/$UpdateName.tgz 1> /dev/null 2> /dev/null
    else
      echo -e -n " [\\033[1;33mAlready Downloaded\\033[0;00m]"
    fi
    echo -e -n " [\\033[1;32mChecking\\033[0;00m]"
    MD5SUM=`cat $TEMPDIR/CHECKSUMS.md5 | grep "$UpdateName.tgz$" | cut -d" " -f1`
    TRUEMD5SUM=`md5sum $TEMPDIR/$TrueUpdateName.tgz | cut -d" " -f1`
    if [ "$MD5SUM" != "$TRUEMD5SUM" ]; then
      echo "$MD5SUM $TRUEMD5SUM"
      echo -e -n " [\\033[1;31mMD5SUM Mismatch\\033[0;00m] [\\033[1;31mDeleting\\033[0;00m]"
      rm $TEMPDIR/$TrueUpdateName.tgz 1> /dev/null 2> /dev/null
    else
      echo -e -n " [\\033[1;32mDone\\033[0;00m]"
    fi
  fi
}

# Installs updates for level 1 (Download Only)
install_level_1() {    
  if [ ! -z "$OUTPUTDIR" ]; then
    copy_output
    INSTDIR=$OUTPUTDIR
  else
    INSTDIR=$TEMPDIR
  fi
  echo "Updates are located in $INSTDIR"
  echo "Install them by using the following command as root:"
  if [ "$Patches" == "slackware" ]; then
    echo -e "  \\033[36;40mupgradepkg $INSTDIR/glibc*.tgz\\033[0;00m"
    echo -e "  \\033[36;40mupgradepkg $INSTDIR/cxx*.tgz\\033[0;00m"
    echo -e "  \\033[36;40mupgradepkg $INSTDIR/sed*.tgz\\033[0;00m"
    echo -e "  \\033[36;40mupgradepkg $INSTDIR/bash*.tgz\\033[0;00m"    
  fi
  echo -e "  \\033[36;40mupgradepkg $INSTDIR/*.tgz\\033[0;00m"
  echo
  echo -e "\\033[36;40mIf you upgraded a kernel make sure you run lilo.\\033[0;00m"
}

# Installs updates for level 2 (Auto Install)
install_level_2() {
  /sbin/upgradepkg *.tgz
  copy_output
  if [ -r /etc/lilo.conf ]; then
    if [ -x /sbin/lilo ]; then
      /sbin/lilo
    fi
  fi
}

# Installs the upgrade for level2
install_level_2_upgrade() {
  /sbin/upgradepkg glibc*.tgz 2> /dev/null
  /sbin/upgradepkg cxx*.tgz 2> /dev/null
  /sbin/upgradepkg sed*.tgz 2> /dev/null
  /sbin/upgradepkg bash*.tgz 2> /dev/null
  install_level_2
  RCD_List=`ls /etc/rc.d/*\.new 2> /dev/null`
  if [ ! -z "$RCD_List" ]; then
    for RCD_New in $RCD_List; do
      RCD_Org=`echo $RCD_New | sed -e "s/\.new//g"`
      mv $RCD_Org $RCD_Org.old
      mv $RCD_New $RCD_Org
    done
  fi
  if [ "$SVersion" == "slackware-current" ]; then
    echo "If this is the first time you are upgrading to slackware-current you may need to run netconfig."
  else
    if [ -x /sbin/netconfig ]; then
      /sbin/netconfig
    fi
  fi
}

# Installs new updates
install_updates() {
  if [ ! -z "`ls $TEMPDIR/*.tgz 2> /dev/null`" ]; then
    NoUpdates=0
  fi
  if [ "$NoUpdates" == "1" ]; then
    echo "No new updates found"
    clean_tempdir
  else
    case $RunLevel in
      0)	clean_tempdir
		;;
      1)	install_level_1
		;;		
      2)	echo "Installing Updates..."
		if [ "$Patches" == "slackware" ]; then
		  install_level_2_upgrade
		else
		  install_level_2
		fi
		;;
      *)	;;
    esac
  fi
}

# Initalize the temp directory
init_dirs() {
  ORGDIR=`pwd`
  clean_tempdir
  if [ ! -d $TEMPDIR ]; then
    mkdir $TEMPDIR
    if [ $? != 0 ]; then
      FAILURE="Could not create directory $TEMPDIR"
      exit_failure
    fi
    cd "$TEMPDIR"
    TEMPDIR=`pwd` 
  fi
}

# Find and report any .new files that are in /etc
look_for_new_files() {
  if [ "$EUID" == "0" ]; then
    if [ ! -z "`find /etc | grep "\.new$" 2> /dev/null`" ]; then
      echo
      echo -e "The following config files have changed:\\033[1;33m"
      find /etc | grep "\.new$" 2> /dev/null
      echo -e "\\033[0;00mYou may need to manually update them."
    fi
  fi
}

# Check to see if there is a package on the server that is newer than one on this machine
match_packages() {
  UpdateNames=`cat $TEMPDIR/CHECKSUMS.md5 | grep ".tgz$" | grep -v "\.\." | sed -e "s/.*\.\///" | sed -e "s/.tgz//"`
  TestUpdateNames=`cat $TEMPDIR/CHECKSUMS.md5 | grep ".tgz$" | grep -v "\.\." | sed -e "s/.*\///g" | sed -e "s/-[0-9A-Z].*//g"`

  if [ "$Patches" == "slackware" ]; then
    PackageNames=`ls /var/log/packages`
    TestPackageNames=`ls /var/log/packages | sed -e "s/-[0-9A-Z].*//g"`
    upgrade_changes
  fi
  
  PackageNames=`ls /var/log/packages`
  TestPackageNames=`ls /var/log/packages | sed -e "s/-[0-9A-Z].*//g"`
  PNamesCount=`echo $PackageNames | wc -w | sed -e "s/\ //g"`
  
  echo "Checking for Updates..."
  pnum=0
  for TestPackageName in $TestPackageNames; do
    if [ "$Verbose" == 1 ]; then
      echo -e -n "Processing package [$((pnum+1)) of $PNamesCount]\\033[99D"
    fi
    unum=0
    foundmatch=0
    for TestUpdateName in $TestUpdateNames; do
    
      if [ "$TestUpdateName" == "$TestPackageName" ]; then

 	if [ $foundmatch == 1 ]; then
	  echo -e "\\033[1;31mWARNING!!! UPDATE FOR THIS PACKAGE ALREADY DOWNLOADED!"
	  echo -e "Please Delete the un-necessary package before installing.\\033[0;00m"
	fi

	PackageName=`echo $PackageNames | cut -d \  -f $(($pnum+1))`
	UpdateName=`echo $UpdateNames | cut -d \  -f $(($unum+1))`
        TrueUpdateName=`echo $UpdateName | sed -e "s/.*\///g"`

	if [ "$TestPackageName" == "kernel-modules" ]; then
	  Modules_PackageName=`echo $PackageName | sed -e "s/_.*//g" | sed -e "s/-i[34]86.*//g"`
	  Modules_TrueUpdateName=`echo $TrueUpdateName | sed -e "s/_.*//g" | sed -e "s/-i[34]86.*//g"`
	  if [ "$Modules_PackageName" == "$Modules_TrueUpdateName" ]; then
	    TrueUpdateName=$PackageName
	  fi
	fi

        if [ "$PackageName" != "$TrueUpdateName" ]; then
	  download_package
	  echo
	  foundmatch=1
        elif [ "$Verbose" ==  1 ]; then
          echo -e -n "$PackageName = $UpdateName [\\033[1;32mUp2Date\\033[0;00m]"
	  echo
        fi
      fi
    
      unum=$((unum+1))
    done
    pnum=$((pnum+1))
  done
  echo -e -n "\\033[2K\n"
}

# Check command line options
parse_options() {
  ExpectTemp=0
  ExpectLevel=0
  ExpectOutput=0
  ExpectServer=0
  ExpectVersion=0
  for OPTION in $OPTIONS; do
    case $OPTION in
    	-h)	echo "$HELP"
		exit_failure
		;;
    	-help)	echo "$HELP"
		exit_failure
		;;
    	--help)	echo "$HELP"
		exit_failure
		;;
    	-\?)	echo "$HELP"
		exit_failure
		;;
	-l)	ExpectLevel=1
		;;
	-o)	ExpectOutput=1
		;;
	-r)	Retry=1
		;;
	-s)	ExpectServer=1
		;;
	-t)	ExpectTemp=1
		;;
	-u)	Patches="slackware"
	        SVersion="slackware-current"
		ExpectVersion=1
		;;
	-v)	Verbose=1
		;;
	*)	if [ "$ExpectTemp" == "1" ]; then
		  TEMPDIR="$OPTION"
		  ExpectTemp=0
		elif [ "$ExpectOutput" == "1" ]; then
		  OUTPUTDIR="$OPTION"
		  ExpectOutput=0
		elif [ "$ExpectLevel" == "1" ]; then
		  RunLevel="$OPTION"
		  ExpectLevel=0
		elif [ "$ExpectServer" == "1" ]; then
		  ServerP="$OPTION"
		  ExpectServer=0
		elif [ "$ExpectVersion" == "1" ]; then
		  SVersion="$OPTION"
		  ExpectVersion=0		
		else
		  echo "$HELP"
		  FAILURE="Incorrect command line parameters."
		  exit_failure
		fi
		;;
    esac
  done
}

# Verify that a correct runlevel has been specified
runlevel_check() {
  echo -n "Entering RunLevel: $RunLevel "
  if [ "$RunLevel" == 2 ]; then
    echo "[Automatically Install Updates]"
    if [ "$EUID" != "0" ]; then
      FAILURE="You must have root privileges to update system."
      exit_failure
    fi
  elif [ "$RunLevel" == 1 ]; then
    echo "[Check for and Download Updates]"
  elif [ "$RunLevel" == 0 ]; then
    echo "[Only Check for Updates]"
  else
    echo "[Unknown]"
    FAILURE="RunLevel Set incorrectly.  Please check Config."
    exit_failure
  fi
}

# Find a server
server_check() {
  echo -n "Checking for a Server"
  Servers="$ServerP $Server1 $Server2 $Server3"
  for TServer in $Servers; do
    if [ ! -r $TEMPDIR/CHECKSUMS.md5 ]; then
      echo -n "."
      wget $wgetOptions $TServer/$SVersion/$Patches/CHECKSUMS.md5 1> /dev/null 2> /dev/null
      Server=$TServer/$SVersion
      ServerOld=$TServer/$SVersionOld
    fi
  done
  if [ ! -r $TEMPDIR/CHECKSUMS.md5 ]; then
    echo -e "[\\033[1;31mFailed\\033[0;00m]"
    FAILURE="No Servers available."
    exit_failure
  else 
    echo -e "[\\033[36;40mDone\\033[0;00m]"
    echo -e "Using \\033[36;40m$Server\\033[0;00m"
  fi
}

# Verify this system's version of Slackware
system_version_check() {
  echo -n "Checking Version..."
  if [ -r /etc/slackware-version ]; then
    Version1=`cat /etc/slackware-version | cut -d" " -f1 | tr 'A-Z' 'a-z'`
    Version2=`cat /etc/slackware-version | cut -d" " -f2 | cut -d. -f1,2`
    SVersionOld="$Version1-$Version2"
  else
    echo
    FAILURE="Could not find /etc/slackware-version.  This system is probably not slackware."
    exit_failure
  fi
  
  if [ "$Patches" != "slackware" ]; then
    SVersion=$SVersionOld
  elif [ "$SVersion" == "slackware-current" ] || [ "$SVersionOld" == "slackware-current" ]; then
    Patches="slackware"
    SVersionOld="slackware-8.1"
  fi
  echo -e "[\\033[36;40m$SVersion\\033[0;00m]"
}

# Create packages that didn't exist in your previous version of slackware
upgrade_changes() {
  cp CHECKSUMS.md5 CHECKSUMS-upgrade.md5
  rm CHECKSUMS.md5
  wget $wgetOptions $ServerOld/$Patches/CHECKSUMS.md5 1> /dev/null 2> /dev/null
  cp CHECKSUMS.md5 CHECKSUMS-old.md5
  rm CHECKSUMS.md5  
  cp CHECKSUMS-upgrade.md5 CHECKSUMS.md5

  cat CHECKSUMS-upgrade.md5 | grep "tgz$" | sed -e "s/.*\///" | cut -d/ -f2 | sed -e "s/-[0-9].*//" > UpgradeList.md5
  cat CHECKSUMS-old.md5 | grep "tgz$" | sed -e "s/.*\///" | cut -d/ -f2 | sed -e "s/-[0-9].*//" > OldList.md5
  UpgradeList=`diff OldList.md5 UpgradeList.md5 | grep ">" | cut -d\  -f2`

  for UPakcage in $UpgradeList; do
    found=0
    for TestPackageName in $TestPackageNames; do
      if [ "$UPakcage" == "$TestPackageName" ]; then
	found=1
      fi
    done
    if [ "$found" == "0" ]; then
      NewUpgradePkgs="$NewUpgradePkgs $UPakcage"
    fi
  done
    
  if [ -w /var/log/packages ]; then
    for NewUpgradePkg in $NewUpgradePkgs; do
      if [ -z "`ls /var/log/packages/$NewUpgradePkg-* 2> /dev/null`" ]; then
	echo "$UPakcage" > /var/log/packages/$NewUpgradePkg-0.0.0-i386-1
      fi      
    done
  else
    echo
    echo "You do not have permission to modify /var/log/packages."
    echo "Please create the following files if packages for them do not exist:"
    for NewUpgradePkg in $NewUpgradePkgs; do
      if [ -z "`ls /var/log/packages/$NewUpgradePkg-* 2> /dev/null`" ]; then
	echo -e " - \\033[1;33m/var/log/packages/$NewUpgradePkg-0.0.0-i386-1\\033[0;00m"
      fi
    done
    echo
  fi
}

###########
# Program #
###########
echo
echo -e "Welcome to \\033[36;40m$VERSION\\033[0;00m"
echo "Initializing..."
check_for_commands
OPTIONS="$@"
parse_options
runlevel_check
init_dirs
check_for_new_version
system_version_check
server_check
match_packages
install_updates
look_for_new_files
echo
echo "Done"
echo -e "\\033[0;00m"
